<?php
/**
 * Moving Contents
 *
 * @package    Moving Contents
 * @subpackage MovingContents Main function
/*  Copyright (c) 2021- Katsushi Kawamori (email : dodesyoswift312@gmail.com)
	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; version 2 of the License.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

$movingcontents = new MovingContents();

/** ==================================================
 * Class Main function
 *
 * @since 1.00
 */
class MovingContents {

	/** ==================================================
	 * Construct
	 *
	 * @since 1.00
	 */
	public function __construct() {

		add_action( 'movingcontents_generate_json_hook', array( $this, 'generate_json' ), 10, 1 );
		add_action( 'movingcontents_update_db_hook', array( $this, 'update_db' ), 10, 5 );
		add_action( 'movingcontents_clear_db_hook', array( $this, 'clear_db' ) );

	}

	/** ==================================================
	 * Gennerate json
	 *
	 * @param string $json_file  json_file.
	 * @since 1.00
	 */
	public function generate_json( $json_file ) {

		/* Scan files */
		global $wpdb;
		$posts = $wpdb->get_results(
			"
			SELECT	*
			FROM	{$wpdb->prefix}posts
			"
		);

		if ( empty( $posts ) ) {
			?>
			<div class="notice notice-error is-dismissible"><ul><li><?php esc_html_e( 'There is no contents.', 'moving-contents' ); ?></li></ul></div>
			<?php
			exit;
		}

		$posts_meta = $wpdb->get_results(
			"
			SELECT	*
			FROM	{$wpdb->prefix}postmeta
			"
		);

		$posts_comment = $wpdb->get_results(
			"
			SELECT	comment_ID, comment_post_ID, comment_author, comment_author_email, comment_author_url, comment_author_IP, comment_date, comment_date_gmt, comment_content, comment_karma, comment_approved, comment_agent, comment_type, comment_parent, user_id
			FROM	{$wpdb->prefix}comments
			LEFT	JOIN {$wpdb->prefix}posts ON( {$wpdb->prefix}comments.comment_post_id = {$wpdb->prefix}posts.ID )
			"
		);

		$term_taxonomy = $wpdb->get_results(
			"
			SELECT	*
			FROM	{$wpdb->prefix}term_taxonomy
			"
		);

		$terms = $wpdb->get_results(
			"
			SELECT	*
			FROM	{$wpdb->prefix}terms
			"
		);

		$termmeta = $wpdb->get_results(
			"
			SELECT	*
			FROM	{$wpdb->prefix}termmeta
			"
		);

		$term_relationships = $wpdb->get_results(
			"
			SELECT	*
			FROM	{$wpdb->prefix}term_relationships
			"
		);

		$post_json = json_encode(
			array(
				'post' => $posts,
				'postmeta' => $posts_meta,
				'comment' => $posts_comment,
				'term_taxonomy' => $term_taxonomy,
				'terms' => $terms,
				'termmeta' => $termmeta,
				'term_relationships' => $term_relationships,
			)
		);

		file_put_contents( $json_file, $post_json );

	}

	/** ==================================================
	 * Update DB
	 *
	 * @param string $json_file  json_file.
	 * @param int    $uid  uid.
	 * @param array  $user_ids  user_ids.
	 * @param string $search_url  search_url.
	 * @param string $replace_url  replace_url.
	 * @since 1.00
	 */
	public function update_db( $json_file, $uid, $user_ids, $search_url, $replace_url ) {

		global $wpdb;

		$json = file_get_contents( $json_file );
		$import_data = json_decode( $json );

		foreach ( $import_data as $key1 => $value1 ) {
			foreach ( $value1 as $key2 => $value2 ) {
				if ( 'post' === $key1 ) {
					$table = $wpdb->prefix . 'posts';
					if ( is_null( get_post( $value2->ID ) ) ) {
						$wpdb->insert( $table, array( 'ID' => $value2->ID ), array( '%d' ) );
					}
					$where = array( 'ID' => $value2->ID );
					if ( array_key_exists( $value2->post_author, $user_ids ) ) {
						$pid = $user_ids[ $value2->post_author ];
					} else {
						$pid = $value2->post_author;
					}
					if ( 0 < $uid ) {
						$value2->post_author = $uid;
					} else {
						$value2->post_author = $pid;
					}
					$wpdb->update( $table, (array) $value2, $where );
				} else if ( 'postmeta' === $key1 ) {
					$table = $wpdb->prefix . 'postmeta';
					if ( empty( get_post_meta( $value2->post_id ) ) ) {
						$wpdb->insert( $table, (array) $value2 );
					}
				} else if ( 'comment' === $key1 ) {
					$table = $wpdb->prefix . 'comments';
					if ( is_null( get_comment( $value2->comment_ID ) ) ) {
						$wpdb->insert( $table, array( 'comment_ID' => $value2->comment_ID ), array( '%d' ) );
					}
					$where = array( 'comment_ID' => $value2->comment_ID );
					if ( array_key_exists( $value2->user_id, $user_ids ) ) {
						$pid = $user_ids[ $value2->user_id ];
					} else {
						$pid = $value2->user_id;
					}
					if ( 0 < $uid && 0 < $value2->user_id ) {
						$value2->user_id = $uid;
					} else {
						$value2->user_id = $pid;
					}
					$wpdb->update( $table, (array) $value2, $where );
				} else if ( 'term_taxonomy' === $key1 ) {
					$table = $wpdb->prefix . 'term_taxonomy';
					if ( ! get_term_by( 'id', $value2->term_id, $value2->taxonomy ) ) {
						$wpdb->insert( $table, (array) $value2 );
					}
				} else if ( 'terms' === $key1 ) {
					$table = $wpdb->prefix . 'terms';
					$is_term_id = false;
					$is_term_id = $wpdb->get_var(
						$wpdb->prepare(
							"
							SELECT term_id
							FROM {$wpdb->prefix}terms
							WHERE term_id = %d
							",
							$value2->term_id
						)
					);
					if ( ! $is_term_id ) {
						$wpdb->insert( $table, (array) $value2 );
					}
				} else if ( 'termmeta' === $key1 ) {
					$table = $wpdb->prefix . 'termmeta';
					$is_meta_id = false;
					$is_meta_id = $wpdb->get_var(
						$wpdb->prepare(
							"
							SELECT meta_id
							FROM {$wpdb->prefix}termmeta
							WHERE term_id = %d
							",
							$value2->term_id
						)
					);
					if ( ! $is_meta_id ) {
						$wpdb->insert( $table, (array) $value2 );
					}
				} else if ( 'term_relationships' === $key1 ) {
					$table = $wpdb->prefix . 'term_relationships';
					$term_taxonomy_ids = array();
					$term_taxonomy_ids = $wpdb->get_col(
						$wpdb->prepare(
							"
							SELECT term_taxonomy_id
							FROM {$wpdb->prefix}term_relationships
							WHERE object_id = %d
							",
							$value2->object_id
						)
					);
					if ( ! in_array( $value2->term_taxonomy_id, $term_taxonomy_ids ) ) {
						$wpdb->insert( $table, (array) $value2 );
					}
				}
			}
		}

		if ( ! empty( $search_url ) && ! empty( $replace_url ) ) {
			/* Replace */
			$wpdb->query(
				$wpdb->prepare(
					"
					UPDATE {$wpdb->prefix}posts
					SET post_content = replace( post_content, %s, %s )
					",
					$search_url,
					$replace_url
				)
			);
		}

		echo '<div class="notice notice-success is-dismissible"><ul><li>' . esc_html__( 'The import is now complete.', 'moving-contents' ) . '</li></ul></div>';

	}

	/** ==================================================
	 * Clear DB
	 *
	 * @since 1.00
	 */
	public function clear_db() {

		global $wpdb;

		$wpdb->query( "DELETE FROM {$wpdb->prefix}posts" );
		$wpdb->query( "DELETE FROM {$wpdb->prefix}postmeta" );
		$wpdb->query( "DELETE FROM {$wpdb->prefix}comments" );
		$wpdb->query( "DELETE FROM {$wpdb->prefix}term_taxonomy" );
		$wpdb->query( "DELETE FROM {$wpdb->prefix}terms" );
		$wpdb->query( "DELETE FROM {$wpdb->prefix}termmeta" );
		$wpdb->query( "DELETE FROM {$wpdb->prefix}term_relationships" );

	}

}


